/*  Pasang Emas. Enjoy a unique traditional game of Brunei.
    Copyright (C) 2015  Nor Jaidi Tuah

    This file is part of Pasang Emas.
      
    Pasang Emas is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

namespace Pasang {

public class PasangApplication : Gtk.Application {

    public PasangApplication () {
        Object (application_id: "bn.pasang-emas", flags: ApplicationFlags.FLAGS_NONE);
    }

    public override void startup () {
        base.startup ();
        create_app_menu ();
    }

    protected override void activate () {
        var main_window = new Gtk.ApplicationWindow (this);
        try {
            main_window.set_icon_from_file (Config.PIXMAPSDIR + "/pasang-emas.png");
        }
        catch (Error e) {
            stderr.printf ("File error: %s\n", e.message);
        }
        main_window.add (new Pasang.Window (main_window));
        main_window.show_all ();
    }

    private void create_app_menu () {
        set_app_menu (MenuMaker.begin (this, "app")
            .add ("Help",   _("_Help"),   on_contents,  "F1")
            .add ("About",  _("_About"),  on_about)
            .add_separator ()
            .add ("Quit",   _("_Quit"),   quit)
        .end ());
   }

    private void on_contents () {
        try {
            Gtk.show_uri (active_window.screen, "help:pasang-emas",
                          Gtk.get_current_event_time ());
        }
        catch (Error e) {
            stderr.printf("Error: %s \n", e.message);
        }
    }

    private void on_about () {
        Gdk.Pixbuf logo = null;
        try {
            logo = new Gdk.Pixbuf.from_file (Config.PIXMAPSDIR + "/pasang-emas-flash.png");
        }
        catch (Error e) {
            stderr.printf ("Cannot load logo: %s\n", e.message);
        }
        Gtk.show_about_dialog (active_window,
            "program-name",  _(Config.PACKAGE_NAME),
            "version", Config.PACKAGE_VERSION,
            "comments", _("A traditional board game of Brunei"),
            "copyright", "Copyright (C) 2008-2015 Nor Jaidi Tuah",
            "logo", logo,
            "authors", new string[]{
                        "Nor Jaidi Tuah <norjaidi.tuah@ubd.edu.bn> (programmer)",
                        "Mohd Abdoh bin Haji Awang Damit (initiator)", null },
            "artists", new string[]{"Nor Jaidi Tuah <norjaidi.tuah@ubd.edu.bn>", null},
            "translator_credits", _("translator_credits"),
            "license-type", Gtk.License.GPL_3_0,
            "website", "http://pasang-emas.sourceforge.net/",
            "website-label", "pasang-emas.sourceforge.net"
        );
    }

}//PasangApplication
}//namespace

// vim: tabstop=4: expandtab: textwidth=100: autoindent:
